@@ -3,6 +3,7 @@
  *  Copyright (C) 2003 Choe Hwanjin(krisna@kldp.org)
  *  Copyright (c) 2006 Remco den Breeje <remco@sx.mine.nu>
  *  Copyright (c) 2008 Diego Ongaro <ongardie@gmail.com>
+ *  Copyright (c) 2016 Landry Breuil <landry@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Library General Public License as published
@@ -38,13 +39,6 @@
 
 #define DATETIME_MAX_STRLEN 256
 
-/* check for new Xfce 4.10 panel features */
-#ifdef LIBXFCE4PANEL_CHECK_VERSION
-#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
-#define HAS_PANEL_49
-#endif
-#endif
-
 /**
  *  Convert a GTimeVal to milliseconds.
  *  Fractions of a millisecond are truncated.
@@ -259,6 +253,7 @@ static gboolean close_calendar_window(t_datetime *datetime)
   gtk_widget_destroy(datetime->cal);
   datetime->cal = NULL;
 
+  xfce_panel_plugin_block_autohide (XFCE_PANEL_PLUGIN (datetime->plugin), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(datetime->button), FALSE);
 
   return TRUE;
@@ -286,7 +281,7 @@ static GtkWidget * pop_calendar_window(t_datetime *datetime, int orientation)
 
   /* set screen number */
   screen = gtk_widget_get_screen(parent);
-  num = gdk_screen_get_monitor_at_window(screen, parent->window);
+  num = gdk_screen_get_monitor_at_window(screen, gtk_widget_get_window(parent));
   gtk_window_set_screen(GTK_WINDOW(window), screen);
 
   frame = gtk_frame_new(NULL);
@@ -297,7 +292,7 @@ static GtkWidget * pop_calendar_window(t_datetime *datetime, int orientation)
   display_options = GTK_CALENDAR_SHOW_HEADING |
     GTK_CALENDAR_SHOW_WEEK_NUMBERS |
     GTK_CALENDAR_SHOW_DAY_NAMES;
-  gtk_calendar_display_options(GTK_CALENDAR (cal), display_options);
+  gtk_calendar_set_display_options(GTK_CALENDAR (cal), display_options);
   gtk_container_add (GTK_CONTAINER(frame), cal);
 
   g_signal_connect_after(G_OBJECT(window), "realize",
@@ -306,8 +301,12 @@ static GtkWidget * pop_calendar_window(t_datetime *datetime, int orientation)
   g_signal_connect_swapped(G_OBJECT(window), "delete-event",
       G_CALLBACK(close_calendar_window),
       datetime);
+  g_signal_connect_swapped(G_OBJECT(window), "focus-out-event",
+      G_CALLBACK(close_calendar_window),
+      datetime);
   gtk_widget_show_all(window);
 
+  xfce_panel_plugin_block_autohide (XFCE_PANEL_PLUGIN (datetime->plugin), TRUE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(datetime->button), TRUE);
 
   return window;
@@ -346,26 +345,66 @@ static gboolean datetime_clicked(GtkWidget *widget,
 
 static void datetime_update_date_font(t_datetime *datetime)
 {
+#if GTK_CHECK_VERSION (3, 16, 0)
+    GtkCssProvider *css_provider;
+    gchar * css;
+#if GTK_CHECK_VERSION (3, 20, 0)
+    css = g_strdup_printf("label { font: %s; }",
+#else
+    css = g_strdup_printf(".label { font: %s; }",
+#endif
+                          datetime->date_font);
+    /* Setup Gtk style */
+    DBG("css: %s",css);
+    css_provider = gtk_css_provider_new ();
+    gtk_css_provider_load_from_data (css_provider, css, strlen(css), NULL);
+    gtk_style_context_add_provider (
+        GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (datetime->date_label))),
+        GTK_STYLE_PROVIDER (css_provider),
+        GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    g_free(css);
+#else
   PangoFontDescription *font;
   font = pango_font_description_from_string(datetime->date_font);
 
   if (G_LIKELY (font))
   {
-    gtk_widget_modify_font(datetime->date_label, font);
+    gtk_widget_override_font(datetime->date_label, font);
     pango_font_description_free (font);
   }
+#endif
 }
 
 static void datetime_update_time_font(t_datetime *datetime)
 {
+#if GTK_CHECK_VERSION (3, 16, 0)
+    GtkCssProvider *css_provider;
+    gchar * css;
+#if GTK_CHECK_VERSION (3, 20, 0)
+    css = g_strdup_printf("label { font: %s; }",
+#else
+    css = g_strdup_printf(".label { font: %s; }",
+#endif
+                          datetime->time_font);
+    /* Setup Gtk style */
+    DBG("css: %s",css);
+    css_provider = gtk_css_provider_new ();
+    gtk_css_provider_load_from_data (css_provider, css, strlen(css), NULL);
+    gtk_style_context_add_provider (
+        GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (datetime->time_label))),
+        GTK_STYLE_PROVIDER (css_provider),
+        GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    g_free(css);
+#else
   PangoFontDescription *font;
   font = pango_font_description_from_string(datetime->time_font);
 
   if (G_LIKELY (font))
   {
-    gtk_widget_modify_font(datetime->time_label, font);
+    gtk_widget_override_font(datetime->time_label, font);
     pango_font_description_free (font);
   }
+#endif
 }
 
 static void datetime_set_update_interval(t_datetime *datetime)
@@ -590,19 +629,13 @@ void datetime_write_rc_file(XfcePanelPlugin *plugin, t_datetime *dt)
 /*
  * change widgets orientation when the panel orientation changes
  */
-#ifdef HAS_PANEL_49
 static void datetime_set_mode(XfcePanelPlugin *plugin, XfcePanelPluginMode mode, t_datetime *datetime)
 {
-  GtkOrientation panel_orientation = xfce_panel_plugin_get_orientation (plugin);
   GtkOrientation orientation = (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL) ?
     GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL;
-#else
-static void datetime_set_orientation(XfcePanelPlugin *plugin, GtkOrientation orientation, t_datetime *datetime)
-{
-#endif
   if (orientation == GTK_ORIENTATION_VERTICAL)
   {
-    xfce_hvbox_set_orientation(XFCE_HVBOX(datetime->box), GTK_ORIENTATION_HORIZONTAL);
+    gtk_orientable_set_orientation(GTK_ORIENTABLE(datetime->box), GTK_ORIENTATION_HORIZONTAL);
     gtk_label_set_angle(GTK_LABEL(datetime->time_label), -90);
     gtk_label_set_angle(GTK_LABEL(datetime->date_label), -90);
     gtk_box_reorder_child(GTK_BOX(datetime->box), datetime->time_label, 0);
@@ -610,7 +643,7 @@ static void datetime_set_orientation(XfcePanelPlugin *plugin, GtkOrientation ori
   }
   else
   {
-    xfce_hvbox_set_orientation(XFCE_HVBOX(datetime->box), GTK_ORIENTATION_VERTICAL);
+    gtk_orientable_set_orientation(GTK_ORIENTABLE(datetime->box), GTK_ORIENTATION_VERTICAL);
     gtk_label_set_angle(GTK_LABEL(datetime->time_label), 0);
     gtk_label_set_angle(GTK_LABEL(datetime->date_label), 0);
     gtk_box_reorder_child(GTK_BOX(datetime->box), datetime->date_label, 0);
@@ -631,7 +664,7 @@ static void datetime_create_widget(t_datetime * datetime)
   gtk_widget_show(datetime->button);
 
   /* create a box which can be easily adapted to the panel orientation */
-  datetime->box = xfce_hvbox_new(GTK_ORIENTATION_VERTICAL, TRUE, 0);
+  datetime->box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
   gtk_widget_show(datetime->box);
   gtk_container_add(GTK_CONTAINER(datetime->button), datetime->box);
@@ -644,20 +677,16 @@ static void datetime_create_widget(t_datetime * datetime)
 
   /* add time and date lines to the box */
   gtk_box_pack_start(GTK_BOX(datetime->box),
-      datetime->time_label, FALSE, FALSE, 0);
+      datetime->time_label, TRUE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(datetime->box),
-      datetime->date_label, FALSE, FALSE, 0);
+      datetime->date_label, TRUE, FALSE, 0);
 
   /* connect widget signals to functions */
   g_signal_connect(datetime->button, "button-press-event",
       G_CALLBACK(datetime_clicked), datetime);
 
   /* set orientation according to the panel orientation */
-#ifdef HAS_PANEL_49
   datetime_set_mode(datetime->plugin, orientation, datetime);
-#else
-  datetime_set_orientation(datetime->plugin, orientation, datetime);
-#endif
 }
 
 /*
@@ -695,10 +724,8 @@ static void datetime_free(XfcePanelPlugin *plugin, t_datetime *datetime)
   /* stop timeouts */
   if (datetime->timeout_id != 0)
     g_source_remove(datetime->timeout_id);
-#if USE_GTK_TOOLTIP_API
   if (datetime->tooltip_timeout_id != 0)
     g_source_remove(datetime->tooltip_timeout_id);
-#endif
 
   /* destroy widget */
   gtk_widget_destroy(datetime->button);
@@ -733,11 +760,7 @@ static void datetime_construct(XfcePanelPlugin *plugin)
       G_CALLBACK(datetime_set_size), datetime);
   g_signal_connect(plugin, "configure-plugin",
       G_CALLBACK(datetime_properties_dialog), datetime);
-#ifdef HAS_PANEL_49
   g_signal_connect(plugin, "mode-changed", G_CALLBACK(datetime_set_mode), datetime);
-#else
-  g_signal_connect(plugin, "orientation-changed", G_CALLBACK(datetime_set_orientation), datetime);
-#endif
   xfce_panel_plugin_menu_show_configure(plugin);
 }
 
